@@ -4,17 +4,17 @@
 #' clusters (k).
 #'
 #' There are three ways to handle the input data before clustering via argument
-#' \code{prep.data}. The default is to use the raw data as-is ("none"). Or, we
-#' can enact \code{\link{prepare_data}} on the full dataset ("full"), or the
-#' bootstrap sampled datasets ("sampled").
+#' `prep.data`. The default is to use the raw data as-is ("none"). Or, we can
+#' enact [prepare_data()] on the full dataset ("full"), or the bootstrap sampled
+#' datasets ("sampled").
 #'
 #' @param cons.funs consensus functions to use. Current options are "kmodes"
 #'   (k-modes), "majority" (majority voting), "CSPA" (Cluster-based Similarity
 #'   Partitioning Algorithm), "LCE" (linkage clustering ensemble)
-#' @param evaluate logical; if \code{TRUE} (default), validity indices are
-#'   returned. Internal validity indices are always computed. If \code{ref.cl}
-#'   is not \code{NULL}, then external validity indices will also be computed.
-#' @param plot logical; if \code{TRUE}, \code{graph_all} is called and a summary
+#' @param evaluate logical; if `TRUE` (default), validity indices are returned.
+#'   Internal validity indices are always computed. If `ref.cl` is not `NULL`,
+#'   then external validity indices will also be computed.
+#' @param plot logical; if `TRUE`, `graph_all` is called and a summary
 #'   evaluation heatmap of ranked algorithms vs. internal validity indices is
 #'   plotted as well.
 #' @inheritParams consensus_cluster
@@ -23,13 +23,13 @@
 #' @inheritParams impute_knn
 #' @return A list with the following elements
 #' \item{E}{raw clustering ensemble object}
-#' \item{Eknn}{clustering ensemble object with knn imputation used on \code{E}}
+#' \item{Eknn}{clustering ensemble object with knn imputation used on `E`}
 #' \item{Ecomp}{flattened ensemble object with remaining missing entries imputed
 #' by majority voting}
 #' \item{clusters}{final clustering assignment from the diverse clustering
 #' ensemble method}
-#' \item{indices}{if \code{evaluate = TRUE}, shows cluster evaluation indices;
-#' otherwise \code{NULL}}
+#' \item{indices}{if `evaluate = TRUE`, shows cluster evaluation indices;
+#' otherwise `NULL`}
 #' @author Aline Talhouk, Derek Chiu
 #' @export
 #' @examples
@@ -44,7 +44,8 @@
 #' "kmodes", ref.cl = ref.cl, progress = FALSE)
 #' str(dice.obj, max.level = 2)
 dice <- function(data, nk, reps = 10, algorithms = NULL, k.method = NULL,
-                 nmf.method = c("brunet", "lee"), distance = "euclidean",
+                 nmf.method = c("scd", "lee"), nmf.loss = c("mse", "mkl"),
+                 distance = "euclidean",
                  cons.funs = c("kmodes", "majority", "CSPA", "LCE"),
                  sim.mat = c("cts", "srs", "asrs"),
                  prep.data = c("none", "full", "sampled"), min.var = 1,
@@ -59,18 +60,21 @@ dice <- function(data, nk, reps = 10, algorithms = NULL, k.method = NULL,
   # Generate Diverse Cluster Ensemble
   E <- consensus_cluster(data = data, nk = nk, reps = reps,
                          algorithms = algorithms, nmf.method = nmf.method,
-                         distance = distance, prep.data = prep.data,
-                         min.var = min.var, progress = progress)
+                         nmf.loss = nmf.loss, distance = distance,
+                         prep.data = prep.data, min.var = min.var,
+                         progress = progress)
   # KNN imputation
   Eknn <- apply(E, 2:4, impute_knn, data = data, seed = seed)
 
   # Select k and new (trimmed and reweighed) data
   if (progress)
     cli::cat_line("Selecting k and imputing non-clustered cases")
+
   eval.obj <- consensus_evaluate(data = data, Eknn, ref.cl = ref.cl,
                                  k.method = k.method, trim = trim,
                                  reweigh = reweigh, n = n)
-  Eknn <- eval.obj$trim.obj$E.new
+  trim.obj <- eval.obj$trim.obj
+  Eknn <- trim.obj$E.new
   k <- eval.obj$k
 
   # Impute remaining missing cases
@@ -79,6 +83,7 @@ dice <- function(data, nk, reps = 10, algorithms = NULL, k.method = NULL,
   # Consensus functions
   if (progress)
     cli::cat_line("Computing consensus functions")
+
   Final <- purrr::map2(Ecomp, k, ~ {
     vapply(cons.funs, function(x) {
       switch(x,
@@ -91,10 +96,9 @@ dice <- function(data, nk, reps = 10, algorithms = NULL, k.method = NULL,
   })
 
   #  If more than one k, need to prepend "k=" labels
-  if (length(Ecomp) > 1) {
+  if (length(Ecomp) > 1)
     Final <- purrr::map2(Final, k,
                          ~ magrittr::set_colnames(.x, paste_k(colnames(.), .y)))
-  }
 
   # Relabel Final Clustering using reference (or first column if no reference)
   clusters <- Final %>%
@@ -106,28 +110,26 @@ dice <- function(data, nk, reps = 10, algorithms = NULL, k.method = NULL,
   if (evaluate) {
     if (progress)
       cli::cat_line("Evaluating output with consensus function results")
-    eval.obj2 <- consensus_evaluate(data, E, cons.cl = clusters,
+
+    eval.obj2 <- consensus_evaluate(data = data, E, cons.cl = clusters,
                                     ref.cl = ref.cl, plot = plot)
-    indices <- c(k = list(eval.obj[["k"]]), eval.obj2[2:4],
-                 trim = list(eval.obj[["trim.obj"]]))
+    indices <- c(k = k, eval.obj2[2:4], trim = list(trim.obj))
   } else {
     indices <- NULL
   }
 
   # Add the reference class as the first column if provided
-  if (!is.null(ref.cl)) {
+  if (!is.null(ref.cl))
     clusters <- cbind(Reference = ref.cl, clusters)
-  }
 
   # Algorithm vs internal index heatmap
-  if (plot) {
+  if (plot)
     algii_heatmap(data, k, E, clusters, ref.cl)
-  }
 
-  # Remove list structure
-  Eknn <- abind::abind(Eknn, along = 3)
-  Ecomp <- abind::abind(Ecomp, along = 3)
+  # Combine DICE with different E and indices
   if (progress)
     cli::cat_line("Diverse Cluster Ensemble Completed")
-  dplyr::lst(E, Eknn, Ecomp, clusters, indices)
+
+  dplyr::lst(E, Eknn, Ecomp, clusters, indices) %>%
+    purrr::map_at(2:3, abind::abind, along = 3) # Unlist Eknn/Ecomp
 }
